package nc.bs.smart.data;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.ufida.dataset.IContext;
import com.ufida.dataset.db.DbType;
import com.ufida.iufo.pub.tools.AppDebug;
import com.ufida.zior.exception.MessageException;
import com.ufida.zior.perfwatch.PerfWatch;

import nc.bs.bi.smart.ConfigDBTool;
import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.framework.common.RuntimeEnv;
import nc.bs.iufo.dao.SQLExec;
import nc.bs.logging.Logger;
import nc.itf.ls.ILightScheduler;
import nc.itf.smart.ISmartService;
import nc.jdbc.framework.JdbcSession;
import nc.jdbc.framework.PersistenceManager;
import nc.jdbc.framework.SQLParameter;
import nc.jdbc.framework.exception.DbException;
import nc.jdbc.framework.processor.ColumnListProcessor;
import nc.jdbc.framework.processor.MapListProcessor;
import nc.jdbc.framework.processor.ResultSetProcessor;
import nc.jdbc.framework.util.DBConsts;
import nc.md.MDBaseQueryFacade;
import nc.md.model.IBean;
import nc.md.model.ITable;
import nc.md.model.MetaDataException;
import nc.mddb.MDDBBaseQueryFacade;
import nc.pub.bi.clusterscheduler.SchedulerUtilities;
import nc.pub.bi.meta.IMeta;
import nc.pub.iufo.cache.CacheManager;
import nc.pub.smart.cache.GlobalConfig;
import nc.pub.smart.cache.GlobalConfigCache;
import nc.pub.smart.cache.SmartConfigCache;
import nc.pub.smart.cache.SmartDefCache;
import nc.pub.smart.cache.SmartModelCache;
import nc.pub.smart.context.SmartContext;
import nc.pub.smart.data.DataSet;
import nc.pub.smart.data.DataSetRequest;
import nc.pub.smart.data.DbTable;
import nc.pub.smart.data.DbTableType;
import nc.pub.smart.data.SmartModelData;
import nc.pub.smart.data.dialect.IDataBaseDialect;
import nc.pub.smart.exception.SmartException;
import nc.pub.smart.metadata.DataTypeConstant;
import nc.pub.smart.metadata.Field;
import nc.pub.smart.metadata.MetaData;
import nc.pub.smart.model.DefaultExecStrategy;
import nc.pub.smart.model.MaterExecStrategy;
import nc.pub.smart.model.SmartModel;
import nc.pub.smart.model.descriptor.ConfigDescriptor;
import nc.pub.smart.model.descriptor.Descriptor;
import nc.pub.smart.provider.impl.procedure.ProcedureRequest;
import nc.pub.smart.provider.impl.procedure.ProcedureResult;
import nc.pub.smart.script.engine.MetaLinkFinder;
import nc.pub.smart.script.engine.ScriptEngine;
import nc.pub.smart.script.engine.event.ProcessOrderByEvent;
import nc.pub.smart.script.expression.LongValue;
import nc.pub.smart.script.expression.operators.relational.EqualsTo;
import nc.pub.smart.script.statement.select.AllColumns;
import nc.pub.smart.script.statement.select.Limit;
import nc.pub.smart.script.statement.select.PlainSelect;
import nc.pub.smart.script.statement.select.Select;
import nc.pub.smart.script.statement.select.SelectBody;
import nc.pub.smart.script.statement.select.SelectItem;
import nc.pub.smart.script.statement.select.SubSelect;
import nc.pub.smart.script.statement.select.Union;
import nc.pub.smart.script.util.deparser.ExpressionDeParser;
import nc.pub.smart.script.util.deparser.StatementDeParser;
import nc.pub.smart.util.DescriptorUtil;
import nc.pub.smart.util.SmartUtilities;
import nc.pub.smart.util.SqlUtil;
import nc.pubitf.para.SysInitQuery;
import nc.vo.bi.clusterscheduler.TempTableVO;
import nc.vo.bi.clusterscheduler.TimeSettingVO;
import nc.vo.jcom.lang.StringUtil;
import nc.vo.ls.TaskResult;
import nc.vo.ml.NCLangRes4VoTransl;
import nc.vo.pub.BusinessRuntimeException;
import nc.vo.pub.lang.UFDateTime;
import nc.vo.pubapp.pattern.exception.ExceptionUtils;
import nc.vo.smart.config.SmartDataSource;
import nccloud.commons.lang.StringUtils;
import oracle.jdbc.OracleTypes;
import uap.bs.ae.datasource.DataSourceManager;
import uap.pub.ae.datasource.DataSourceCache;
import uap.pub.ae.datasource.temptable.TempTableManager;
import uap.vo.ae.procedure.ParameterType;
import uap.vo.ae.procedure.ProcedureParameter;

/**
 * ģͷ
 *
 * @author liuyy
 *
 */
public class SmartService implements ISmartService {

	/**
	 * NCԪʵ
	 */
	private static final String DB_MAINENTITY_TAG = "DBMainEntity";

	/**
	 * ݼΪݿһʱ һдڴݿ
	 *
	 * @param dataset
	 *            ״̬Ϊopen;
	 * @param table
	 *            ԪԼͣtableΪգ
	 * @return
	 */
	public DbTable storeDataSet(DataSet dataset, DbTable table)
			throws Exception {
		if (dataset.getMetaData() == null && table.getDsName() == null)
			throw new IllegalArgumentException(
					"MetaData or datasource name  is Null");

		table.setMetaData(dataset.getMetaData());

		if (table.getTablename() == null) {
			table.setTablename(DbTable.genTempName());
		}

		// ֤б
		if (SmartUtilities.getDDCService().isTableExisted(table)) {
			table.setTablename(DbTable.genTempName());
		}

		PerfWatch pw = new PerfWatch("SmartService.storeDataSet(): "
				+ table.getTablename());
		try {
			pw.appendMessage(table);
			pw.appendMessage(" rowCount:" + dataset.getCount());
			
			//wby 20220722 д룬ʱָдԴ
			String adgDsName = table.getDsName();
			if(adgDsName.toUpperCase().contains("ADG")) {
				String wDsName = "NCCDATA";
				try {
					wDsName = SysInitQuery.getParaString("GLOBLE00000000000000", "ADGParam01");
					if(StringUtils.isEmpty(wDsName)) {
						wDsName = adgDsName;
					}
					
				} catch (Exception e) {
					ExceptionUtils.wrappException(e);
				}
				table.setDsName(wDsName);//ʱָдⴴʱ
			}
			
			SmartUtilities.getDDCService().createTable(table);
			
			//wby 20220722 ԴΪADG 򴴽ʱдȴͬADGݿ
			if(adgDsName.toUpperCase().contains("ADG")) {
				Integer second = SysInitQuery.getParaInt("GLOBLE00000000000000", "ADGWaitTim");
				if (second != null && second > 0) {
					Thread.sleep(second * 1000);
				} else {
					Thread.sleep(1000);
				}
				
				//wby 20220726 ʱҪָض ֧DDLɾģ
				table.setDsName(adgDsName);
			}
			
//			SQLExec.execBatch(
//					table.getDsName(),
//					SqlUtil.getInsertSQLWithParam(dataset.getMetaData(),
//							table.getTablename(),
//							SmartUtilities.getDbType(table.getDsName())),
//					SqlUtil.getSQLParam(dataset));
			execBatch(table.getDsName(), SqlUtil.getInsertSQLWithParam(dataset.getMetaData(),table.getTablename(),SmartUtilities.getDbType(table.getDsName())),SqlUtil.getSQLParam(dataset));
			
		} catch (Exception e) {
			pw.appendMessage(e);
			throw e;
		} finally {
			pw.stop();
		}
		return table;
	}
	
	/*
	 * wby 20220728 д Ҫύٵȴͬ
	 */
	private int execBatch(String dsName, String sql, SQLParameter[] params) {
		 if (dsName == null) {
			 throw new IllegalArgumentException("SQLExec: dsName is null.");
		 }
		 
		 dsName = getPhysicalDS(dsName);
		 
		 if (sql == null || sql.length() < 1) {
			 return 0;
		 }
		 
		 PersistenceManager persist = null;
		 JdbcSession jdbc = null;
		 
		 try {
			 persist = DataSourceManager.getInstance().getPersistenceManager(dsName);
			 persist.setAddTimeStamp(false);
			 jdbc = persist.getJdbcSession();
			 jdbc.setSQLTranslator(false);
			 jdbc.addBatch(sql, params);
			 
			 if(dsName.toUpperCase().contains("ADG")) {
				 jdbc.addBatch("commit");
			 }
			 return jdbc.executeBatch();
		 } catch (Exception e) {
			 AppDebug.error(e);
			 throw new RuntimeException(NCLangRes4VoTransl.getNCLangRes().getStrByID("1413002_0", "01413002-0964") + e.getMessage(), e);
		 } finally {
			 if (jdbc != null) {
				 jdbc.closeAll(); 
			 }
			 if (persist != null) {
				 persist.release();
			 }
		 }
	}
	
	/*
	 * wby 20220728 д
	 */
	private static String getPhysicalDS(String dsName) { return DataSourceCache.getPhysicalDSName(dsName); }
	
	public DbTable storeDataSet_RequiresNew(DataSet dataset, DbTable table)
			throws Exception {
		return storeDataSet(dataset,table);
	}

	// /**
	// * ȡ liuyy.
	// *
	// * @param metaData
	// * @param sql
	// * @param dsName
	// * @return
	// */
	// public Object[][] fetchData(final MetaData metaData, String sql,
	// String dsName) {
	// if (metaData == null || sql == null || dsName == null) {
	// throw new IllegalArgumentException();
	// }
	// Object obj = new Object[0][];
	// final PerfWatch pw = new PerfWatch("SmartService.fetchData()");
	// pw.appendMessage("dsName: " + dsName).appendMessage("sql: " + sql);
	// try {
	// obj = SQLExec.execQuery(dsName, sql, new ResultSetProcessor() {
	// private static final long serialVersionUID = 1L;
	//
	// public Object handleResultSet(ResultSet rs) throws SQLException {
	// // ݲѯֶΣΪеڣԪеֶζѯliuyy
	// // 2009-08-14
	// Field[] columns = getColumnNamesByResultSet(rs);
	//
	// List<Field> listFlds = new ArrayList<Field>();
	// // ҪȡԪеλ
	// final Map<String, Integer> hashIndex = new HashMap<String, Integer>();
	// for (Field col : columns) {
	// Field f = metaData.getField(col.getFldname());
	// if (f != null) {
	// listFlds.add(f);
	// hashIndex.put(f.getFldname(), metaData.getIndex(f));
	// }
	// }
	// List<Object[]> rowList = new ArrayList<Object[]>();
	// int metaDataFieldNum = metaData.getFieldNum();
	//
	// while (rs.next()) {
	// Object[] data = new Object[metaDataFieldNum];
	// for (Field field : listFlds) {
	// Object value = null;
	// try {
	// value = rs.getObject(field.getFldname());
	// value = SmartUtilities.convert2TargetType(
	// field.getDataType(), value);
	// int index = hashIndex.get(field.getFldname());
	// data[index] = value;
	// } catch (Exception e) {
	// AppDebug.debug(e);
	// }
	// }
	// rowList.add(data);
	// }
	// Object[][] datas = rowList.toArray(new Object[0][]);
	// pw.appendMessage("FetchData Count: " + datas.length);
	// return datas;
	//
	// }
	// });
	// } catch (Exception e) {
	// pw.appendMessage(e);
	// throw new RuntimeException(e.getMessage(), e);
	// } finally {
	// pw.stop();
	// }
	//
	// return (Object[][]) obj;
	//
	// }

	/**
	 * ȡԪϢ liuyy.
	 *
	 * @param sql
	 * @param dsName
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("deprecation")
	public MetaData fetchMetaData(String sql, String dsName) throws Exception {

		if (sql == null || sql.trim().length() == 0) {
			throw new IllegalArgumentException("sql is null.");
		}

		DbType dbType = SmartUtilities.getDbType(dsName);
		// ߽űģͽ
		Select select = null;
		try {
			select = (Select) ScriptEngine.parse(sql);
		} catch (Exception e) {
			select = null;
		}
		if (select != null) {
			SelectBody sb = select.getSelectBody();
			PlainSelect ps = null;
			if (sb instanceof PlainSelect) {
				ps = (PlainSelect) sb;
			} else if (sb instanceof Union) {
				Union union = (Union) sb;
				ps = new PlainSelect();
				// ----selectֶ--------
				List<SelectItem> selectItemList = new Vector<SelectItem>();
				selectItemList.add(new AllColumns());
				ps.setSelectItems(selectItemList);

				// --------from ----------
				SubSelect subSelect = new SubSelect();
				subSelect.setSelectBody(union);
				subSelect.setAlias("nestQuery");
				ps.setFromItem(subSelect);

				select.setSelectBody(ps);
			}else{
				ps = new PlainSelect();
			}

			// where1=0
			if(dbType == DbType.GBASE){
				Limit limit = new Limit();
				limit.setRowCount(1);
				ps.setLimit(limit);
			}else{
				EqualsTo equal = new EqualsTo();
				equal.setLeftExpression(new LongValue(1));
				equal.setRightExpression(new LongValue(0));
				ps.addWhereExpression(equal);
			}

			// 
			StringBuffer sbf = new StringBuffer();
			StatementDeParser dp = new StatementDeParser(sbf);
			dp.visit(select, new ExpressionDeParser());
			sql = sbf.toString();
		} else {
			// ַǶ select * from (ԭʼsql) temp where 1=0
			if(dbType == DbType.GBASE){
				sql = "select * from (" + sql + ") temp limit 1";
			}else{
				sql = "select * from (" + sql + ") temp where 1=0";
			}

		}

		Object obj = new Object[0];

		try {
			/* ʱȡ߼ */
			if (StringUtils.indexOf(sql, DbTable.PREX_TEMP_TABLE) > 0) {
				obj = TempTableManager.getTempTableManager().execQuery(dsName, sql, new ResultSetProcessor() {
					private static final long serialVersionUID = -4156999551501797136L;

					public Object handleResultSet(ResultSet rs) throws SQLException {
						return SmartResultSetProcessor.getColumnNamesByResultSet(rs);
					}
				}, true);
			} else {
				obj = SQLExec.execQuery(dsName, sql, new ResultSetProcessor() {
					private static final long serialVersionUID = 3660339080876010242L;

					public Object handleResultSet(ResultSet rs) throws SQLException {
						return SmartResultSetProcessor.getColumnNamesByResultSet(rs);
					}
				}, true);
			}
		} catch (DbException dbe) {
			AppDebug.error("Error sql: " + sql);
			AppDebug.error(dbe);

			// ݿ쳣̨¼־ֱӷװmessageԭʼ쳣
			throw new SmartException("DataBase access error:"
					+ dbe.getMessage());
		} catch (Exception e) {
			AppDebug.error("Error sql: " + sql);
			throw new SmartException(e.getMessage(), e);
		} 
		
		MetaData md = new MetaData();
		md.addField((Field[]) obj);
		return md;

	}

	/**
	 * ȡ
	 *
	 * @throws SmartException
	 */
	public DataSet fetchDataSet(String sql, String dsName)
			throws SmartException {
		DataSetRequest request = new DataSetRequest(sql, dsName);
		return fetchDataSet(request);
	}

	/**
	 * ȡ
	 *
	 * metadataݼ漰 ֶ˳ ȡmetadata Ϊ ѭԭ
	 * --ֶ˳metadataУsqlûе ȥ metadataûУsqlе 棻
	 * --תmetadataת
	 *
	 * @throws Exception
	 */
	public DataSet fetchDataSet(DataSetRequest request) throws SmartException {

		final PerfWatch pw = new PerfWatch(
				"SmartService.fetchDataSet(DataSetRequest)");
		pw.appendMessage("maxRows: " + request.getMaxRows());

		String dsName = request.getDsName();
		String sql = request.getSql();
		if (dsName == null || dsName.trim().length() < 1) {
			dsName = SmartUtilities.getCurrentDsName();
		}

		if (sql == null || dsName == null) {
			throw new IllegalArgumentException();
		}
		Object obj = new Object[0][];
		pw.appendMessage("dsName: " + dsName).appendMessage("sql: " + sql);

		// ű
		if (request.isEnableScriptRule()) {
			try {
				Select select = (Select) ScriptEngine.parse(sql);

				SmartContext context = new SmartContext();
				context.setDsName4Exec(dsName);
				ProcessOrderByEvent evt = new ProcessOrderByEvent(select,
						context);

				select = ScriptEngine.handleEvent(evt);

				// sql
				sql = ScriptEngine.select2Sql(select);
			} catch (Throwable e) {
				String message = "SmartService.fetchDataSet():catch exception when process sql with scriptRule:"
						+ e.getMessage();
				Logger.error(message);
				pw.appendMessage(message);
			}
		}

		SmartResultSetProcessor rsProcessor = null;
		
		//չʹչ
		String clzName = request.getRsProcessorClzName();
		if(clzName != null){
			try {
				Class<?> cls = Class.forName(clzName);
				Constructor<?> cons = cls.getConstructor(DataSetRequest.class);
				Object extProcessor = cons.newInstance(request);
				if(extProcessor != null && extProcessor instanceof SmartResultSetProcessor){
					rsProcessor = (SmartResultSetProcessor)extProcessor;
				}
			} catch (ClassNotFoundException e) {
				AppDebug.error(e.getMessage());
			} catch (NoSuchMethodException e) {
				AppDebug.error(e.getMessage());
			} catch (Exception e) {
				AppDebug.error(e.getMessage());
			}finally{
				if(rsProcessor == null){
					rsProcessor = new SmartResultSetProcessor(request);
				}
			}
		}else{
			rsProcessor = new SmartResultSetProcessor(request);
		}
		
		try {
			/*ʱȡ߼*/
			if (StringUtils.indexOf(sql, DbTable.PREX_TEMP_TABLE) > 0) {
				obj = TempTableManager.getTempTableManager().execQuery(dsName, sql,
						rsProcessor, request.isEnableSQLTranslator());
			} else {
				obj = SQLExec.execQuery(dsName, sql, rsProcessor,
						request.isEnableSQLTranslator());
			}
		} catch (DbException dbe) {
			pw.appendMessage("Error sql: " + sql);
			pw.appendMessage(dbe);
			AppDebug.error("Error sql: " + sql);
			AppDebug.error(dbe);

			// ݿ쳣̨¼־ֱӷװmessageԭʼ쳣
			throw new SmartException("DataBase access error:"
					+ dbe.getMessage());
		} catch (Exception e) {
			pw.appendMessage("Error sql: " + sql);
			pw.appendMessage(e);
			AppDebug.error("Error sql: " + sql);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}

		return (DataSet) obj;

	}

	/**
	 * ȡݱ
	 *
	 * @param table
	 * @return
	 * @throws Exception
	 */
	public DataSet fetchDataSet(DbTable table) throws SmartException {
		if (table == null || table.getMetaData() == null) {
			throw new IllegalArgumentException();
		}

		DataSetRequest request = new DataSetRequest(table.getMetaData(),
				table.toSelectSQL(), table.getDsName(), 0, true, false);
		DataSet dataset = fetchDataSet(request);

		return dataset;

	}

	

	

	/**
	 * ȡ
	 *
	 * @param sql
	 *            ԭʼselect sql
	 * @param dsName
	 *            Դ
	 * @return
	 * @throws Exception
	 */
	public int fetchRecordCount(String sql, String dsName) throws Exception {
		// SQL Server Ӳѯвorder by ---biancm 20090828
		DbType dbType = SmartUtilities.getDbType(dsName);
		if (dbType == DbType.SQLSERVER) {
			sql = SqlUtil.removeOrderBy(sql);
		}

		String countSql = "select count(1) num from (" + sql + ") temp_Count"; // sqlcountװ
		// try {
		return SQLExec.getRecordCount(dsName, countSql);
		// } catch (Exception e) {
		// throw new RuntimeException(e.getMessage(), e);
		// }

	}

	@Override
	public Integer[] provideCount(SmartModelData[] models)
			throws SmartException {
	  Logger.error("SmartService is in : [" + RuntimeEnv.getInstance().getNCHome() + "]");
		if (models == null)
			return new Integer[0];

		int size = models.length;
		Integer[] rslt = new Integer[size];
		PerfWatch pw = new PerfWatch("SmartService.provideCount()");
		try {
			for (int i = 0; i < size; i++) {
				SmartModelData data = models[i];
				SmartModel model = data.getModel();
				if (model == null) {
					continue;
				}
				rslt[i] = model.provideCount(data.getContext(),
						data.getDescriptors());
			}
		} catch (BusinessRuntimeException e) {
			pw.appendMessage(e);
			throw e;
		} catch (SmartException e) {
			pw.appendMessage(e);
			throw e;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}
		return rslt;
	}

	@Override
	public String[] provideSQL(SmartModelData[] models) throws SmartException {
		if (models == null)
			throw new IllegalArgumentException();
		int size = models.length;
		String[] rslt = new String[size];

		PerfWatch pw = new PerfWatch("SmartService.provideSQL()");
		try {
			for (int i = 0; i < size; i++) {
				SmartModelData data = models[i];
				SmartModel model = data.getModel();
				if (model == null) {
					continue;
				}
				rslt[i] = model.provideSQL(data.getContext(),
						data.getDescriptors());
			}
		} catch (BusinessRuntimeException e) {
			pw.appendMessage(e);
			throw e;
		} catch (SmartException e) {
			pw.appendMessage(e);
			throw e;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}
		return rslt;
	}

	@Override
	public DataSet[] provideData(SmartModelData[] models) throws SmartException {
		if (models == null)
			return new DataSet[0];

		PerfWatch pw = new PerfWatch("SmartService.provideData()");

		int size = models.length;
		DataSet[] rslt = new DataSet[size];
		pw.appendMessage("SmartModelData[]: " + size);
		try {
			for (int i = 0; i < size; i++) {
				SmartModelData data = models[i];
				SmartModel model = data.getModel();
				if (model == null) {
					continue;
				}
				rslt[i] = model.provideData(data.getContext(),
						data.getDescriptors());
			}
		} catch (BusinessRuntimeException e) {
			pw.appendMessage(e);
			throw e;
		} catch (SmartException e) {
			pw.appendMessage(e);
			throw e;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}
		return rslt;
	}

	@Override
	public DbTable[] provideView(SmartModelData[] models) throws SmartException {
		if (models == null)
			return new DbTable[0];

		PerfWatch pw = new PerfWatch("SmartService.provideView()");

		int size = models.length;
		DbTable[] rslt = new DbTable[size];
		try {
			for (int i = 0; i < size; i++) {
				SmartModelData data = models[i];
				SmartModel model = data.getModel();
				if (model == null) {
					continue;
				}
				rslt[i] = model.provideView(data.getContext(),
						data.getDescriptors());
			}
		} catch (BusinessRuntimeException e) {
			pw.appendMessage(e);
			throw e;
		} catch (SmartException e) {
			pw.appendMessage(e);
			throw e;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}

		return rslt;
	}

	/**
	 * ṩԪMetaData ע⣺ģɵsqlءֱȡģ͵Ԫ ڲʹáⲿƼʹ
	 *
	 * @param models
	 * @return
	 * @throws SmartException
	 */
	public MetaData[] provideMetaData(SmartModelData[] models)
			throws SmartException {
		if (models == null)
			throw new IllegalArgumentException();
		int size = models.length;
		MetaData[] rslt = new MetaData[size];

		PerfWatch pw = new PerfWatch("SmartService.provideMetaData()");
		try {
			for (int i = 0; i < size; i++) {
				SmartModelData data = models[i];
				SmartModel model = data.getModel();
				if (model == null) {
					continue;
				}

				DefaultExecStrategy execStrategy = new DefaultExecStrategy(
						model);
				String sql = execStrategy.provideSQL(data.getContext(),
						data.getDescriptors());
				String dsName = execStrategy.provideDsName4Exec(null);
				MetaData md = fetchMetaData(sql, dsName);
				rslt[i] = md;
			}
		} catch (BusinessRuntimeException e) {
			pw.appendMessage(e);
			throw e;
		} catch (SmartException e) {
			pw.appendMessage(e);
			throw e;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new SmartException(e.getMessage(), e);
		} finally {
			pw.stop();
		}
		return rslt;
	}

	@Override
	public DataSet execScript(String script, String dsName, IContext context)
			throws SmartException {
		SmartContext smartContext = new SmartContext(context);
		//ִԴ
		try {
			if(smartContext.getDsName4Exec() == null){
				smartContext.setDsName4Exec(dsName);
			}
		} catch (Exception e) {
			AppDebug.error(e);
		}
		
		String sql = ScriptEngine.buildSQL(script, smartContext);
		return fetchDataSet(sql, dsName);
	}

	// /**
	// * ﻯģ
	// *
	// * @param pk
	// * @param table Ϊգʹģﻯ
	// */
	// public void storeSmartModel(String pk, DbTable table){
	// SmartModel model = SmartModelCache.getInstance().getModel(pk);
	// if(model == null){
	// return ;
	// }
	//
	// String tableName =
	// model.getPreferences().getConfigItem().getMaterTableName();
	// String dsName = model.getDsName();
	// DbTable dt = new DbTable(dsName, tableName, DbTable.TYPE_TABLE);
	//
	// SmartUtilities.getSmartService().dropTable(dt);
	//
	// DbTable view = model.provideView(null, null);
	// dt.setMetaData(view.getMetaData());
	//
	// }
	//

	/**
	 * ݹʽȡĿֶ
	 *
	 * @param model
	 *            ģ
	 * @param smartscripts
	 *            ʽ
	 * @return ҲֶΣnull
	 */
	@Override
	public Field[] fetchField(SmartModel model, String... smartScripts) {
		if (smartScripts == null || smartScripts.length < 1) {
			return null;
		}

		Field[] fields = new Field[smartScripts.length];
		for (int i = 0; i < smartScripts.length; i++) {
			String script = smartScripts[i];
			Field f = MetaLinkFinder.findField(model, script);
			fields[i] = f;
		}
		return fields;
	}

	/**
	 * ͻ˻ȡȫû
	 */
	public GlobalConfig getGlobalConfig() {
		return GlobalConfigCache.getGlobalConfig();
	}

	/**
	 * ģԴ
	 *
	 * @param dataSource
	 * @throws Exception
	 */
	public void saveSmartDataSource(SmartDataSource dataSource)
			throws Exception {
		//added by biancm 20130529 Դģл
		try {
			String originalDesignDS = SmartUtilities.getDefDsName();
			String currentDesignDS = dataSource.getDesignDsName();
			if (originalDesignDS != null && currentDesignDS != null
					&& !originalDesignDS.equalsIgnoreCase(currentDesignDS)) {
				CacheManager.refresh();
			}
		} catch (Exception e) {

		}


		// »GlobalConfigCache
		GlobalConfigCache.getSmartConfig().setDataSource(dataSource);

		// 浽ļ
//		IBaseSPService service = NCLocator.getInstance().lookup(
//				IBaseSPService.class);
//		service.saveSmartDataSource(dataSource);
		(new ConfigDBTool()).saveSmartDataSource(dataSource);
	}

	/**
	 * ֶ첽ִﻯ
	 *
	 * @param smartId
	 */
	public void asyncExecMater(String smartId) {
		SmartMaterialTask.asyncExecMater(smartId);
	}

	/**
	 * ִģﻯ
	 *
	 * @param smartId
	 *            ģid
	 * @throws Exception
	 */
	public DbTable execMater(String smartId) throws Exception {
		String code = SmartDefCache.getInstance().getNameWithCode(smartId);
		PerfWatch pw = new PerfWatch("MaterExecStrategy.execMater: " + code);
		SmartModel model = SmartModelCache.getInstance().getModel(smartId);
		if (model == null) {
			pw.appendMessage(nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
					.getStrByID("8001002_0", "08001002-0148", null,
							new String[] { smartId })/*
													 * @res "ģ(" + smartId
													 * +")Ϊգ"
													 */);
		}
		String tableName = null;
		MaterExecStrategy exeStr = new MaterExecStrategy(model);
		if (!exeStr.haveMaterDef()) {
			pw.appendMessage(nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
					.getStrByID("1413002_0", "01413002-1023")/* @res "ﻯΪ" */);
			tableName = DbTable.genTempName();
		} else {
			tableName = model.getPreferences().getConfigItem().getMaterConfig()
					.getTableName();

		}

		try {
			// context
			// 
			SmartContext context = model.getInitialContext();

			// added by zhangyth 2012-12-10
			// ִﻯǰֵΪգ򽫲Ƴ
			String[] names = context.getAttributeNames();
			for (String name : names) {
				if (name.startsWith("parameter_")) {
					Object value = context.getAttribute(name);
					if ("".equals(value)) {
						context.removeAttribute(name);
					}
				}
			}
			// end

			// ﻯԴ
			String dsName = exeStr.provideDsName4Exec(null);

			String sql = new DefaultExecStrategy(model).provideSQL(context,
					null);
			// ģִԴ
			String current_exeDsName = new DefaultExecStrategy(model)
					.provideDsName4Exec(null);

			DbTable srcTable = DbTable.generateTempSql(model.getMetaData(),
					sql, current_exeDsName);

			DbTable dt = new DbTable(dsName, tableName, DbTableType.TABLE);
			SmartUtilities.getDDCService().copyTable(srcTable, dt);
			//add by luaidi ¼ִʱ bi_schd_timer
			String scheduleid = (String)model.getExtProp("scheduleid");
			List<TaskResult>  taskResults = null;
			UFDateTime lasttime = null;
			ILightScheduler schedule =  NCLocator.getInstance().lookup(ILightScheduler.class);
			//scheduleidߵ 
			if(scheduleid != null && !StringUtils.isEmpty(scheduleid)) {
				taskResults = schedule.getResults(scheduleid);
				if(taskResults != null && taskResults.size()>0) {
					//ȡʱ
					lasttime =  taskResults.get(taskResults.size()-1).getEndTime();
				}
				
				//ȡTimeSettingVO 
				TimeSettingVO  timesetvo = new TimeSettingVO();
				String dydsName = SmartConfigCache.getDsName4Design();
				timesetvo = SchedulerUtilities.loadTimeSetting(smartId, dydsName);
				timesetvo.setLastTime(String.valueOf(lasttime));
				SchedulerUtilities.addTimeSetting(timesetvo,dydsName);
			}
			return dt;
		} catch (Exception e) {
			pw.appendMessage(e);
			throw new MessageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("1413002_0", "01413002-1024")/*
																			 * @res
																			 * "ִﻯ"
																			 */
					+ e.getMessage(), e);
		} finally {
			pw.stop();

		}
	}

	@Override
	public DataSet fetchAdhocDataSet(IContext context, IMeta meta,
			String linkModuleId, Descriptor[] des) throws SmartException {

		// IMetaתSmartModel
		SmartModel model = SmartUtilities.meta2Model(meta);

		// SmartContext
		SmartContext smartContext = new SmartContext(context);
		smartContext.setDsName4Exec(meta.getDsName());
		smartContext.setAttribute(SmartContext.KEY_CURR_LINK_MODULE_ID,
				linkModuleId);
		smartContext.setAttribute(SmartContext.KEY_CURR_META, meta);

		// ֱӵģ͵provideDataԪ˳⣩
		// return model.provideData(smartContext, des);

		String sql = model.provideSQL(smartContext, des);

		// ȡAdhocDataSet
		// 
		int maxRows = 0;
		ConfigDescriptor config = DescriptorUtil.getDescriptorByType(
				ConfigDescriptor.class, des);
		if (config != null) {
			maxRows = config.getMaxRows();
			maxRows = maxRows > 0 ? maxRows : model.getPreferences().getConfigItem().getMaxRows();
		}

		DataSetRequest request = new DataSetRequest(sql, meta.getDsName());
		request.setMetadata(meta.getMetaData());
		request.setMaxRows(maxRows);
		DataSet dataSet = fetchDataSet(request);
		return dataSet;
	}

	@Override
	public IBean getEntityByTableID(String tableID) throws SmartException {
		IBean bean;
		try {
			ITable table = null;
			table = MDDBBaseQueryFacade.getInstance().getTableByID(tableID);

			String pkColumnID = table.getPrimaryKey().getPKColumn().getID();
			bean = getEntityByPKColumnID(pkColumnID, tableID);// tableӦʵ
		} catch (Exception e) {
			throw new SmartException("load entity by table error:"
					+ e.getMessage(), e);
		}
		return bean;
	}

	/**
	 * ݱpkNCԪʵ
	 *
	 * @param pkColumnID
	 * @param tableID
	 * @return
	 */
	private IBean getEntityByPKColumnID(String pkColumnID, String tableID) {
		// ñӦʵĻȡһ
		String sql = "select distinct classid from md_ormap where columnid = '"
				+ pkColumnID + "'";
		String beanID = null;
		try {
			List<String> idList = (List<String>) (new BaseDAO().executeQuery(
					sql, new ColumnListProcessor()));
			if (idList != null && idList.size() > 0) {
				if (idList.size() > 1) {
					beanID = getMainEntityFromTable(idList);
				} else {
					beanID = idList.get(0);
				}
			} else {
				String sqlTable = "select distinct classid from md_ormap where tableid = '"
						+ tableID + "'";
				List<String> idListForTable = (List<String>) (new BaseDAO()
						.executeQuery(sqlTable, new ColumnListProcessor()));
				if (idListForTable != null && idListForTable.size() > 0) {
					if (idListForTable.size() > 1) {
						beanID = getMainEntityFromTable(idListForTable);
					} else {
						beanID = idListForTable.get(0);
					}
				}
			}
		} catch (DAOException e1) {
			Logger.error("вѯʵ壬ѯʧܣid" + pkColumnID + " ###@@# " + e1);
		}
		if (StringUtil.isEmptyWithTrim(beanID)) {
			return null;
		}
		IBean bean = null;
		try {
			bean = MDBaseQueryFacade.getInstance().getBeanByID(beanID);
		} catch (MetaDataException e) {
			Logger.error("ѯԪʧܣԪʵid" + beanID + e);
			return null;
		}
		return bean;
	}

	/**
	 * ʵ
	 *
	 * @param beanIDlist
	 * @return
	 * @throws DAOException
	 */
	private String getMainEntityFromTable(List<String> beanIDlist)
			throws DAOException {

		StringBuffer inSql = new StringBuffer("'" + beanIDlist.get(0) + "'");
		for (int i = 1; i < beanIDlist.size(); i++) {
			inSql.append(", '" + beanIDlist.get(i) + "'");
		}
		String sql = "select id,help from md_class where id in (" + inSql + ")";
		List<Map<String, String>> result = (List<Map<String, String>>) new BaseDAO()
				.executeQuery(sql, new MapListProcessor());
		for (Map<String, String> value : result) {
			if (value != null && !StringUtil.isEmptyWithTrim(value.get("help"))
					&& value.get("help").contains(DB_MAINENTITY_TAG)) {
				return value.get("id");
			}
		}
		return beanIDlist.get(0);
	}

	/**
	 * 2013-11-29 ӣɾʱ
	 * @see TempTableSchdTaskdropTempTables1totalMin
	 * @param dsName ԴƣΪ׳SmartExceptionǰԴҪⲿ
	 * @throws SmartException
	 */
	public void dropTempTable(String dsName) throws SmartException{
		if(StringUtils.isEmpty(dsName)){
			throw new SmartException(nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID("0301002_0","00301002-0002")/*@res "ɾʱ쳣ԴΪ"*/);
		}

		//ȡԴdsNameϵʱ1000û
		TempTableVO[] vos = SmartUtilities.getDDCService()
				.getTempTableNames(dsName, 1000);
		if (vos == null || vos.length < 1) {
			return;
		}

		//PerfWatch
		PerfWatch pw = new PerfWatch("SmartService.dropTempTable():" + dsName);
		pw.setModule("SmartService");

		//ɾ
		List<String> sqls = new ArrayList<String>();

		final long timeouts = 10;//ɾʱ䵽ڼ10ӵʱ

		for (TempTableVO vo : vos) {
			try {
				String tableName = vo.getTablename();
				String[] arr =  tableName.split("_");
				if(arr == null || arr.length < 3){
					continue;
				}
				try {
					String createTime = arr[arr.length - 1];
					Calendar calendar = Calendar.getInstance();
					SimpleDateFormat fmt = (SimpleDateFormat) new SimpleDateFormat(
							"HHmm");
					String time = fmt.format(calendar.getTime());

					int old = totalMin(createTime);
					int curr = totalMin(time);
					int diff = curr - old;
					if(diff < 0){
						diff += 24 * 60;
					}

					if (diff < timeouts) {
						continue;
					}

				} catch (Exception e) {
					continue;
				}

				String sql = "drop " + vo.getTableType() + " "
						+ vo.getTablename();
				sqls.add(sql);

			} catch (Exception e) {
				pw.appendMessage(e);
			}
		}

		//ִɾ
		try {
			//JdbcSessionеִѾԶʵ˷ҳύĹ
			SQLExec.execBatch(dsName, sqls.toArray(new String[sqls.size()]));
		} catch (Exception e) {
			pw.appendMessage(sqls);
			throw new SmartException(nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID("0301002_0","00301002-0003")/*@res "ɾʱ쳣"*/+ (StringUtils.isEmpty(e.getMessage()) ? e : e.getMessage()));
		}
	}

	/**
	 * HHmmʽʱ䣬תΪԷΪλʱ
	 * @see this.dropTempTable()
	 * @param flag
	 * @return HH x 60 + mm
	 */
	private int totalMin(String flag){//HHmm
		int hour0 = new Integer(flag.substring(0, 2));
		int min0  = new Integer(flag.substring(2, 4));
		int total = hour0 * 60 + min0;

		return total;
	}

	@Override
	public ProcedureResult callProcedure(ProcedureRequest procedureRequest) throws SmartException {
		
		ProcedureResult procedureResult = new ProcedureResult();
		
		String dsName = procedureRequest.getProcedure().getDsName();
		ProcedureParameter[] parameters = procedureRequest.getProcedure().getParameters();

		PersistenceManager persist = null;
		CallableStatement stmt = null;
		try {
			dsName = DataSourceCache.getPhysicalDSName(dsName);
			persist = DataSourceManager.getInstance().getPersistenceManager(dsName);
			//ȡݿ
			DbType dbType = getDbType(persist);
			//ʱݿProcedureݿͲƥ䣬Procedureݿ
			if(dbType != procedureRequest.getProcedure().getDbType() && dbType != DbType.UNKNOWN) {
				procedureRequest.getProcedure().setDbType(dbType);
			}
			String script = procedureRequest.getProcedure().getScript();
			stmt = persist.getJdbcSession().getConnection().prepareCall(script);
			
			// ò,ųRETURN
			if(parameters != null) {
				int i = 0;
				for(ProcedureParameter parameter: parameters) {
					ParameterType type = parameter.getType();
 					if(type == ParameterType.IN || type == ParameterType.INOUT) {
						Object value = procedureRequest.getParameterValue(parameter.getName());
						setInParamValue(parameter, i+1, value, stmt);
					}
					if(type == ParameterType.OUT || type == ParameterType.INOUT) {
						registOutParam(parameter, i+1, stmt, dbType);
					}
					if(type != ParameterType.RETURN) {
						i++;
					}
				}
			}
			
			// ִд洢
			stmt.execute();
			// ÷ֵ
			ResultSet resultSet = stmt.getResultSet();
			if(resultSet != null) {
				procedureResult.setResultDataSet(
						(DataSet) new SmartResultSetProcessor(new DataSetRequest(null, null)).handleResultSet(resultSet)
					);
			}
			
			// ȡֵ
			if(parameters != null) {
				int i = 0;
				for(ProcedureParameter parameter: parameters) {
					ParameterType type = parameter.getType();
					if(type == ParameterType.OUT || type == ParameterType.INOUT){
						Object result = getOutParamValue(parameter, i+1, stmt, dbType);
						//Outֵ
						procedureResult.getOutParamValueMap().put(parameter.getName(), result);
					}
					if(type != ParameterType.RETURN) {
						i++;
					}
				}
			}
		} catch (DbException e) {
			throw new SmartException(e.getMessage());
		} catch (SQLException e) {
			throw new SmartException(e.getMessage());
		} catch (Exception e) {
			throw new SmartException(e.getMessage());
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
					//do nothing
				}
			}
			if (persist != null) {
				persist.release();
			}
		}
		
		return procedureResult;
	}
	
	private DbType getDbType(PersistenceManager persist) {
		DbType type;
		int iDBType = persist.getDBType();
		switch (iDBType) {
			case DBConsts.DB2:
				type = DbType.DB2;
				break;
			case DBConsts.ORACLE:
				type = DbType.ORACLE;
				break;
			case DBConsts.SQLSERVER:
				type = DbType.SQLSERVER;
				break;
			case DBConsts.HSQL:
				type = DbType.HSQLDB;
				break;
			case DBConsts.POSTGRESQL:
				type = DbType.POSTGRESQL;
				break;
			case DBConsts.SYBASE:
				type = DbType.SYBASE;
				break;
			case DBConsts.GBASE:
				type = DbType.GBASE;
				break;
			case DBConsts.ALCEDO:
				type = DbType.ALCEDO;
				break;
			default:
				type = DbType.UNKNOWN;
		}
		return type;
	}
	
	private Object getOutParamValue(ProcedureParameter parameter, int index, CallableStatement stmt, DbType dbType) throws SmartException {
		
		Object result = null;
		try{
			Method method;
			if(dbType == DbType.ORACLE && parameter.getSqlType() == OracleTypes.CURSOR) {
				method = stmt.getClass().getMethod("getCursor", int.class);
			}else {
				method = CallableStatement.class.getMethod(parameter.genGetter(), int.class);
			}
			result = method.invoke(stmt, index);
		} catch (IllegalAccessException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (IllegalArgumentException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (InvocationTargetException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (NoSuchMethodException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (SecurityException e) {
			throw new SmartException(e.getMessage(), e);
		}
		
		//
		if(result instanceof ResultSet) {
			DataSet dataSet;
			try {
				dataSet = (DataSet) new SmartResultSetProcessor(new DataSetRequest(null, null)).handleResultSet((ResultSet) result);
				result = dataSet;
			} catch (SQLException e) {
				throw new SmartException(e.getMessage());
			}
		}
		
		return result;
	}
	
	private void registOutParam(ProcedureParameter parameter, int index, CallableStatement stmt, DbType dbType) throws SmartException {
		int sqlType = parameter.getSqlType();
		if(sqlType == Types.OTHER && parameter.getTypeName().equals("REF CURSOR") && dbType == DbType.ORACLE) {
			sqlType = OracleTypes.CURSOR;
		}
		
		try {
			Method method;
			method = CallableStatement.class.getMethod("registerOutParameter", int.class, int.class);
			method.invoke(stmt, index, sqlType);
		} catch (NoSuchMethodException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (SecurityException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (IllegalAccessException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (IllegalArgumentException e) {
			throw new SmartException(e.getMessage(), e);
		} catch (InvocationTargetException e) {
			throw new SmartException(e.getMessage(), e);
		}
		
	}
	
	private void setInParamValue(ProcedureParameter parameter, int index, Object value, CallableStatement stmt) throws SmartException {
		
		Method method;
		if(value == null || value.equals("")) {
			//ֵΪյ
			try {
				stmt.setNull(index, parameter.getSqlType());
			} catch (SQLFeatureNotSupportedException e) {
				// ֵ֧setNull
				throw new SmartException(e.getMessage());
			} catch (SQLException e) {
				throw new SmartException(e.getMessage());
			}
		}else {
			try {
				method = PreparedStatement.class.getMethod(parameter.genSetter(), int.class, parameter.getJavaType());
				method.invoke(stmt, index, value);
			} catch (NoSuchMethodException e) {
				throw new SmartException(e.getMessage(), e);
			} catch (SecurityException e) {
				throw new SmartException(e.getMessage(), e);
			} catch (IllegalAccessException e) {
				throw new SmartException(e.getMessage(), e);
			} catch (IllegalArgumentException e) {
				throw new SmartException(e.getMessage(), e);
			} catch (InvocationTargetException e) {
				throw new SmartException(e.getMessage(), e);
			}
		}
		
	}
	@SuppressWarnings("serial")
	private static class DataSetResultSetProcessor implements ResultSetProcessor {
		
		@Override
		public Object handleResultSet(ResultSet resultSet) throws SQLException {
			return resultSet2DataSet(resultSet);
		}
		
		private DataSet resultSet2DataSet(ResultSet rs) throws SQLException {
			Field[] new_fields = getColumnNamesByResultSet(rs);

			MetaData meta = new MetaData(new_fields);

			DataSet dataset = fetchDatas(rs, meta);

			return dataset;
		}

		private DataSet fetchDatas(ResultSet rs, MetaData meta) throws SQLException {
			List<Object[]> rowList = new ArrayList<Object[]>();

			Field[] fields = meta.getFields();

			int metaDataFieldNum = fields.length;

			while (rs.next()) {
				Object[] data = new Object[metaDataFieldNum];
				for (int i = 0; i < metaDataFieldNum; i++) {
					Field field = fields[i];
					Object value = null;

					if (field.getDataType() == DataTypeConstant.OBJECT) {
						value = getBlobValue(rs, field.getFldname());
					} else {
						value = rs.getObject(i + 1);
					}

					value = SmartUtilities.convert2TargetType(field.getDataType(), value);

					// added by zhangyth ȣģеľΪ׼롣
					if (DataTypeConstant.isNumberType(field.getDataType()) && field.getScale() > 0) {
						try {
							value = processScale(value, field.getScale());
						} catch (Exception e) {
							AppDebug.error(e.getMessage());
						}
					}

					data[i] = value;
				}
				rowList.add(data);
			}

			Object[][] datas = rowList.toArray(new Object[0][]);
			DataSet ds = new DataSet(meta, datas);

			return ds;
		}

		private Object processScale(Object value, int scale) {

			if (value == null) {
				return null;
			}

			if (value instanceof BigDecimal) {
				return ((BigDecimal) value).setScale(scale, BigDecimal.ROUND_HALF_UP);
			} else {
				if (value instanceof Double) {
					value = BigDecimal.valueOf((Double) value);
					value = ((BigDecimal) value).setScale(scale, BigDecimal.ROUND_HALF_UP);
					return Double.valueOf(value.toString());
				} else if (value instanceof Float) {
					value = BigDecimal.valueOf((Float) value);
					value = ((BigDecimal) value).setScale(scale, BigDecimal.ROUND_HALF_UP);
					return Float.valueOf(value.toString());
				}
			}

			return value;
		}

		/**
		 * blobֵ
		 * 
		 * @param rs
		 * @param fieldName
		 * @return л
		 */
		private Serializable getBlobValue(ResultSet rs, String fieldName) {
			return null;
		}

		private static Field[] getColumnNamesByResultSet(ResultSet rs) throws SQLException {
			ResultSetMetaData rsmd = rs.getMetaData();
			int iColCount = rsmd.getColumnCount();
			List<Field> fieldList = new ArrayList<Field>();
			for (int i = 0; i < iColCount; i++) {
				int sqlType = rsmd.getColumnType(i + 1);
				String typeName = rsmd.getColumnTypeName(i + 1);

				// DB2ݴDECFLOAT
				if (sqlType == Types.OTHER && typeName.equalsIgnoreCase("decfloat")) {
					sqlType = Types.DECIMAL;
				}

				// ȥҳsqlек
				if (rsmd.getColumnName(i + 1).equalsIgnoreCase(IDataBaseDialect.ROW_NUMBER)) {
					continue;
				}

				Field fld = new Field();
				fieldList.add(fld);
				fld.setDataType(DataTypeConstant.translateType(sqlType));
				fld.setDbColumnType(sqlType);

				//ֶ
				String fldName = rsmd.getColumnName(i + 1);
				String label = rsmd.getColumnLabel(i + 1);
				if (label != null && label.trim().length() > 0 && !label.equalsIgnoreCase(fldName)) {
					fldName = label;// Щʵlabelnameһ¡ݴ
				}
				fld.setFldname(fldName);

				int precision = rsmd.getPrecision(i + 1);
				fld.setPrecision(precision);

				int scale = rsmd.getScale(i + 1);
				fld.setScale(scale);
			}
			return fieldList.toArray(new Field[fieldList.size()]);
		}

	}
	@Override
	public uap.vo.ae.procedure.ProcedureResult callProcedure1(
			uap.vo.ae.procedure.ProcedureRequest procedureRequest)
			throws SmartException {
		uap.vo.ae.procedure.ProcedureResult procedureResult = new uap.vo.ae.procedure.ProcedureResult();
		
		String dsName = procedureRequest.getProcedure().getDsName();
		ProcedureParameter[] parameters = procedureRequest.getProcedure().getParameters();

		PersistenceManager persist = null;
		CallableStatement stmt = null;
		try {
			dsName = DataSourceCache.getPhysicalDSName(dsName);
			persist = DataSourceManager.getInstance().getPersistenceManager(dsName);
			//ȡݿ
//			DbType dbType = getDbType(persist);
			DbType dbType = SmartUtilities.getDbType(dsName);
			//ʱݿProcedureݿͲƥ䣬Procedureݿ
			if(dbType != procedureRequest.getProcedure().getDbType() && dbType != DbType.UNKNOWN) {
				procedureRequest.getProcedure().setDbType(dbType);
			}
			String script = procedureRequest.getProcedure().getScript();
			stmt = persist.getJdbcSession().getConnection().prepareCall(script);
			
			// ò,ųRETURN
			if(parameters != null) {
				int i = 0;
				for(ProcedureParameter parameter: parameters) {
					ParameterType type = parameter.getType();
 					if(type == ParameterType.IN || type == ParameterType.INOUT) {
						Object value = procedureRequest.getParameterValue(parameter.getName());
						setInParamValue(parameter, i+1, value, stmt);
						i++;
					}
					if(type == ParameterType.OUT || type == ParameterType.INOUT) {
						registOutParam(parameter, i+1, stmt, dbType);
						i++;
					}
				}
			}
			
			// ִд洢
			stmt.execute();
			// ÷ֵ
			ResultSet resultSet = stmt.getResultSet();
			if(resultSet != null) {
				procedureResult.setResultDataSet(
						(DataSet) new DataSetResultSetProcessor().handleResultSet(resultSet)
					);
			}
			
			// ȡֵ
			if(parameters != null) {
				for(int i=0;i<parameters.length;i++){
					ProcedureParameter parameter = parameters[i];
					ParameterType type = parameter.getType();
					if(type == ParameterType.OUT || type == ParameterType.INOUT){
						Object result = getOutParamValue(parameter, i+1, stmt, dbType);
						//Outֵ
						procedureResult.getOutParamValueMap().put(parameter.getName(), result);
					}
				}
			}
		} catch (DbException e) {
			throw new SmartException(e.getMessage());
		} catch (SQLException e) {
			throw new SmartException(e.getMessage());
		} catch (Exception e) {
			throw new SmartException(e.getMessage());
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
					//do nothing
				}
			}
			if (persist != null) {
				persist.release();
			}
		}
		
		return procedureResult;
	}
}
