package nccloud.impl.gl.voucherapi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import nc.bs.logging.Logger;
import nc.itf.gl.voucher.IVoucher;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import nc.bs.dao.BaseDAO;
import nc.bs.framework.common.InvocationInfoProxy;
import nc.bs.framework.common.NCLocator;
import nc.bs.framework.comn.NetStreamContext;
import nc.jdbc.framework.processor.MapListProcessor;
import nc.service.rest.utils.ConvertVoucherUtil;
import nc.service.rest.utils.TranslateVoucherUtil;
import nc.vo.gl.pubvoucher.VoucherVO;
import nc.vo.pub.BusinessException;
import nccloud.bs.gl.voucher.VoucherWebBO;
import nccloud.dto.gl.lightgl.VoucherMDVO;
import nc.bs.framework.server.ISecurityTokenCallback;
import nc.bs.framework.server.util.KeyUtil;
import nc.bs.framework.mx.thread.ThreadTracer;
import nccloud.itf.gl.voucherapi.IVoucherRestService;

/**
 * NCCNC63ƾ֤ӿServiceʵ 
 * 2020-11-25
 * 
 * @version NCC2005
 */
public class VoucherRestServiceImpl implements IVoucherRestService {

	private BaseDAO dao = new BaseDAO();
	public static ExecutorService executor = new ThreadPoolExecutor(10, 10, 60L, TimeUnit.SECONDS,
			new ArrayBlockingQueue<Runnable>(10)); // ̳߳

	@Override
	@SuppressWarnings("unchecked")
	public Map<String, Object> tallyBySingleThread(String[] pks, String date, boolean op, boolean isList)
			throws BusinessException {
		if (pks == null || pks.length == 0) {
			throw new BusinessException("ƾ֤Ϊ");
		}

		// 1.׼Ʒˡ
		Map<String, Object> returnMap = null;
		List<VoucherMDVO> successVOs = new ArrayList<VoucherMDVO>(); // ˻ȡƾ֤VO
		synchronized (this) { // ׼Ʒˡ˲дͬһܲ
			returnMap = (new VoucherWebBO()).tallyVoucher(pks, date, op, isList);
			if (returnMap.containsKey("success")) {
				successVOs = (List<VoucherMDVO>) returnMap.get("success");
			}
		}

		// 2.ˡ˺¼
		if (successVOs.size() == 0) {
			return returnMap;
		}
		String[] successKeys = successVOs.stream().map(vo -> vo.getPk_voucher()).toArray(String[]::new); // ˻ȡƾ֤
		IVoucher vservice = NCLocator.getInstance().lookup(IVoucher.class);
		VoucherVO[] vos = vservice.queryByPks(successKeys); // ѯƾ֤VO
		if (vos == null || vos.length == 0) {
			throw new BusinessException("ǰѱɾˢҳ");
		}
		if (op) {
			tallyAfterEvent(Arrays.asList(vos)); // ˺¼
		} else {
			unTallyAfterEvent(Arrays.asList(vos)); // ˺¼
		}
		return returnMap;
	}

	@Override
	@SuppressWarnings("unchecked")
	public Map<String, Object> tallyByMultiThread(String[] pks, String date, boolean op, boolean isList)
			throws BusinessException {
		if (pks == null || pks.length == 0) {
			throw new BusinessException("ƾ֤Ϊ");
		}

		// 1.ȡǰ߳
		Map<String, Serializable> invocationInfo = buildInvocationMap();

		// 2.ݵݸĴС̶߳
		List<Callable<Map<String, Object>>> threadList = new ArrayList<Callable<Map<String, Object>>>(); // ̶߳
		List<List<String>> voucherGroups = groupByVoucher(Arrays.asList(pks)); // ƾ֤VOз
		for (List<String> voucherGroup : voucherGroups) {
			PushVoucherThread thread = new PushVoucherThread(voucherGroup.toArray(new String[0]), date, op, isList); // ߳
			thread.setInvocationInfo(invocationInfo); // ߳
			threadList.add(thread); // ̶߳
		}

		// 3.ִ̣߳ϲؽٷ
		Map<String, Object> result = new HashMap<String, Object>(); // ִ߳н
		try {
			List<Future<Map<String, Object>>> returnData = executor.invokeAll(threadList); // ȴִ߳ٷؽ
			// ϲִ߳н
			for (Future<Map<String, Object>> data : returnData) {
				Map<String, Object> resMap = data.get();

				Object success = resMap.get("success"); // ˻ȡ˳ɹVO
				if (success != null) {
					if (result.containsKey("success")) {
						List<VoucherMDVO> succVOs = (List<VoucherMDVO>) result.get("success");
						succVOs.addAll((List<VoucherMDVO>) success);
						result.put("success", succVOs);
					} else {
						result.put("success", (List<VoucherMDVO>) success);
					}
				}

				Object warn = resMap.get("warn"); // ʾϢ
				if (warn != null) {
					if (result.containsKey("warn")) {
						Map<String, Object> warnMap = (Map<String, Object>) result.get("warn");
						warnMap.putAll((Map<String, Object>) warn);
						result.put("warn", warnMap);
					} else {
						result.put("warn", (Map<String, Object>) warn);
					}
				}

				Object error = resMap.get("error"); // Ϣ
				if (error != null) {
					if (result.containsKey("error")) {
						Map<String, String> errResult = (Map<String, String>) result.get("error");
						errResult.putAll((Map<String, String>) error);
						result.put("error", errResult);
					} else {
						result.put("error", (Map<String, String>) error);
					}
				}
			}

		} catch (Exception e) {
			throw new BusinessException("NCӿʧܣ" + e.getMessage());
		}
		return result;
	}
	
	/**
	 * ƾ֤
	 * 
	 * @param pks
	 * @return
	 * @throws BusinessException
	 */
	public List<List<String>> groupByVoucher(List<String> pks) throws BusinessException {

		if (pks == null || pks.size() == 0) {
			throw new BusinessException("ƾ֤VOΪգ޷");
		}
		int length = pks.size(); // ƾ֤
		int count = ((length - 1) / 10) + 1; // ÿеƾ֤
		int gorupCount = ((length - 1) / count) + 1; // 

		List<List<String>> result = new ArrayList<List<String>>(); // 
		if (gorupCount == 1) {
			result.add(pks);
			return result;
		}
		for (int i = 0; i < gorupCount; i++) {
			List<String> list = new ArrayList<String>();
			if (i == gorupCount - 1) { // һ
				list.addAll(pks.subList(i * count, length));
			} else {
				list.addAll(pks.subList(i * count, (i + 1) * count));
			}
			result.add(list);
		}
		return result;
	}

	/**
	 * ˺¼ƾ֤NC63
	 * 
	 * @param vos
	 * @throws BusinessException
	 */
	public void tallyAfterEvent(List<VoucherVO> vos) throws BusinessException {

		// 1.ƾ֤VOеɱ
		TranslateVoucherUtil translator = new TranslateVoucherUtil();
		translator.translateKey2Code(vos);

		// 2.NC63ӿƾ֤
		Map<String, String> config = getAPIInfo("NC-NCC-GL-001"); // ȡӿϢ
		String url = config.get("url");
		String esbKey = config.get("esbkey");

		Map<String, String> result = new HashMap<String, String>(); // ӿڷ(key:£value:NC63ƾ֤)
		try {
			String json = ConvertVoucherUtil.voucherVOToJsonString(vos); // VOתJSONַ
			JSONObject res = sendJsonPost(url, json, esbKey);
			handleExceptions(res); // 쳣
			JSONArray array = res.getJSONArray("data"); // ƾ֤
			if (array == null || array.size() == 0) {
				throw new BusinessException("NC63صƾ֤Ÿʽ:" + res.toJSONString());
			}
			for (int i = 0; i < array.size(); i++) {
				JSONObject obj = array.getJSONObject(i);
				String year = obj.getString("year"); // 
				String period = obj.getString("period"); // ڼ䣨£
				String free1 = obj.getString("free1"); // 
				String free2 = obj.getString("free2"); // ݺ
				String free3 = obj.getString("free3"); // NC63дƾ֤
				String nccVoucherNO = obj.getString("num"); // NCCƾ֤
				String key = year + "_" + period + "_" + free1 + "_" + free2 + "_" + nccVoucherNO;
				result.put(key, free3);
			}
		} catch (Exception e) {
			throw new BusinessException("NCӿʧܣ" + e.getMessage());
		}

		// 3.ӿڷصNC63ƾ֤ŻдNCCƾ֤ͷ
		for (VoucherVO vo : vos) {
			String year = vo.getYear(); // 
			String period = vo.getPeriod(); // ڼ䣨£
			String free1 = vo.getFree2(); // 
			String free2 = vo.getFree3(); // ݺ
			int nccVoucherNO = vo.getNo(); // ƾ֤

			String key = year + "_" + period + "_" + free1 + "_" + free2 + "_" + nccVoucherNO;
			String free3 = result.get(key); // NC63дƾ֤
			if (free3 == null || "".equals(free3.trim())) {
				throw new BusinessException("NCдƾ֤ʱݴ");
			}
			updateNCVoucherNO(vo.getPk_voucher(), free3); // дNC63ƾ֤
		}
	}

	/**
	 * ˺NCӿɾƾ֤
	 * 
	 * @throws BusinessException
	 */
	public void unTallyAfterEvent(List<VoucherVO> vos) throws BusinessException {

		// 1.˲
		TranslateVoucherUtil translator = new TranslateVoucherUtil();
		Set<String> accBookSet = vos.stream().map(vo -> vo.getPk_accountingbook()).collect(Collectors.toSet());
		Map<String, String> accBookMap = translator.transAccBook(accBookSet, 2); // ֻ跭˲
		for (VoucherVO vo : vos) {
			vo.setPk_accountingbook(accBookMap.get(vo.getPk_accountingbook())); // õVO
		}

		// 2.װNC63ɾƾ֤ӿڲ
		List<Map<String, String>> params = new ArrayList<Map<String, String>>();
		for (VoucherVO vo : vos) {
			Map<String, String> param = new HashMap<String, String>();
			param.put("free1", vo.getFree2() == null ? "" : vo.getFree2()); // 
			param.put("free2", vo.getFree3() == null ? "" : vo.getFree3()); // ݺ
			param.put("free3", vo.getFree4() == null ? "" : vo.getFree4()); // NC63ƾ֤
			param.put("period", vo.getPeriod() == null ? "" : vo.getPeriod()); // ڼ䣨·ݣ
			param.put("year", vo.getYear() == null ? "" : vo.getYear()); // 
			param.put("pk_accbook", vo.getPk_accountingbook()); // ˲
			params.add(param);
		}

		// 3.NC63ɾƾ֤ӿ
		Map<String, String> config = getAPIInfo("NC-NCC-GL-002"); // ȡӿϢ
		String url = config.get("url");
		String esbKey = config.get("esbkey");

		JSONObject json = new JSONObject();
		json.put("data", params);
		try {
			JSONObject result = sendJsonPost(url, json.toJSONString(), esbKey);
			handleExceptions(result); // 쳣
		} catch (Exception e) {
			throw new BusinessException("NCӿʧܣ" + e.getMessage());
		}

		// 4.NCCƾ֤ͷ"NC63ƾ֤"ֶ
		for (VoucherVO vo : vos) {
			updateNCVoucherNO(vo.getPk_voucher(), ""); // NC63ƾ֤
		}
	}

	/**
	 * дNC63ƾ֤ŵNCCƾ֤ͷ Ǽ˺дƾ֤ͷԭĽӿڲ֧״̬ƾ֤update
	 * 
	 * @param pk_voucher NCCƾ֤
	 * @param voucherNO  NC63дƾ֤
	 */
	public void updateNCVoucherNO(String pk_voucher, String voucherNO) throws BusinessException {
		String sql = "update gl_voucher set free4 = '" + voucherNO + "' where pk_voucher = '" + pk_voucher + "'";
		this.dao.executeUpdate(sql);
	}

	/**
	 * post
	 * 
	 * @param url
	 * @param paramJson
	 * @param esbKey
	 * @throws BusinessException
	 */
	public JSONObject sendJsonPost(String url, String paramJson, String esbKey) throws Exception {
		OutputStreamWriter out = null;
		BufferedReader in = null;
		StringBuffer result = new StringBuffer();
		try {
			URL realUrl = new URL(url);
			// 򿪺URL֮
			URLConnection conn = realUrl.openConnection();
			// ͨõ
			conn.setRequestProperty("accept", "*/*");
			conn.setRequestProperty("connection", "Keep-Alive");
			conn.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
			// ַ ַֹԭ򣨷UTF-8µýӿڲһ±Ͳһ£
			conn.setRequestProperty("Content-Type", "application/json;charset=UTF-8");
			conn.setRequestProperty("esbkey", esbKey); // EsbKey,NC63EsbKey֤ӿ
			// POST
			conn.setDoOutput(true);
			conn.setDoInput(true);
			// ȡURLConnectionӦ
			out = new OutputStreamWriter(conn.getOutputStream(), "UTF-8");
			// 
			out.write(paramJson);
			// flushĻ
			out.flush();
			// BufferedReaderȡURLӦ
			in = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
			String line;
			while ((line = in.readLine()) != null) {
				result.append(line);
			}
		} catch (Exception e) {
			throw e;
		} finally { // ʹfinallyر
			try {
				if (out != null) {
					out.close();
				}
				if (in != null) {
					in.close();
				}
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		if (result != null && result.length() > 0) { // תΪJson
			return JSONObject.parseObject(result.toString());
		} else {
			return null;
		}
	}

	/**
	 * NCӿڷرеҵ쳣Ϣ
	 * 
	 * @param json
	 * @throws BusinessException
	 */
	public void handleExceptions(JSONObject json) throws BusinessException {
		String code = json.get("code") == null ? "" : json.get("code").toString();
		String success = json.get("success") == null ? "" : json.get("success").toString();
		if (!"200".equals(code) || !"true".equals(success)) {
			String errorMsg = json.get("message") == null ? "δ֪" : json.get("message").toString();
			throw new BusinessException("NC63ӿڷصĴϢ" + errorMsg);
		}
	}

	/**
	 * ȡǰ߳Ϣ
	 * 
	 * @return
	 */
	private Map<String, Serializable> buildInvocationMap() {
		Map<String, Serializable> result = new HashMap<>();
		InvocationInfoProxy proxy = InvocationInfoProxy.getInstance();
		result.put("_CallId", proxy.getCallId());
		result.put("_BizCenterCode", proxy.getBizCenterCode());
		result.put("_BizDateTime", proxy.getBizDateTime());
		try {
			String busiAction = ThreadTracer.getInstance().getCurThreadEntry().getBusiAction();
			result.put("_BusiAction", busiAction);
		} catch (Exception e) {
			Logger.error(e);
		}
		result.put("_CallPath", proxy.getCallPath());
		result.put("_CallServer", proxy.getCallServer());
		result.put("_DeviceId", proxy.getDeviceId());
		result.put("_GroupId", proxy.getGroupId());
		result.put("_GroupNumber", proxy.getGroupNumber());
		result.put("_HyCode", proxy.getHyCode());
		result.put("_LangCode", proxy.getLangCode());
		result.put("_LogLevel", proxy.getLogLevel());
		result.put("_RunAs", proxy.getRunAs());
		result.put("_Sysid", proxy.getSysid());
		result.put("_TimeZone", proxy.getTimeZone());
		result.put("_UserCode", proxy.getUserCode());
		result.put("_UserDataSource", proxy.getUserDataSource());
		result.put("_UserId", proxy.getUserId());
		result.put("_SessionId", proxy.getProperty("sessionId"));
		byte[] tokenBytes = NetStreamContext.getToken();
		String token = tokenBytes == null ? "default" : KeyUtil.encodeToken(tokenBytes);
		result.put("_token", token);
		return result;
	}

	/**
	 * ȡӿϢ
	 * 
	 * @param code
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public Map<String, String> getAPIInfo(String code) throws BusinessException {

		String sql = "select e.isstart,e.url,c.esbkey from bgy_esbinfo e left join bgy_certificate c on e.certificate = "
				+ " c.pk_code where e.code = '" + code + "'";
		List<Map<String, String>> res = (List<Map<String, String>>) this.dao.executeQuery(sql, new MapListProcessor());
		if (res == null || res.size() != 1) {
			throw new BusinessException("ӿϢ󣬵ӿڰȫȨڵάΪ" + code + "Ϣ");
		}
		String url = res.get(0).get("url"); // NC63ӿڵַ
		if (url == null || "~".equals(url.trim()) || "".equals(url.trim())) {
			throw new BusinessException("ӿURLΪգӿڰȫȨڵάΪ" + code + "Ϣ");
		}
		Map<String, String> result = new HashMap<String, String>();
		result.put("url", url);

		String esbKey = res.get(0).get("esbkey"); // ݿеEsbKey
		result.put("esbkey", (esbKey == null || "".equals(esbKey)) ? "~" : esbKey);
		return result;
	}

}

/**
 * <b> ƾ֤˺ͷ߳ </b> 
 * <b> ˷ʱƾ֤϶̴߳ </b> 
 * 
 * 2020-12-14
 */
class PushVoucherThread implements Callable<Map<String, Object>> {

	String[] pks; // ƾ֤
	private String date; // 
	private boolean op; // true:ˣfalse:
	private boolean isList; // Ƿбҳ

	private Map<String, Serializable> invocationInfo; // ߳
	private IVoucherRestService service = NCLocator.getInstance().lookup(IVoucherRestService.class); // ƾ֤Service

	public PushVoucherThread() {
	}

	public PushVoucherThread(String[] pks, String date, boolean op, boolean isList) {
		this.pks = pks;
		this.date = date;
		this.op = op;
		this.isList = isList;
	}

	public Map<String, Serializable> getInvocationInfo() {
		return invocationInfo;
	}

	public void setInvocationInfo(Map<String, Serializable> invocationInfo) {
		this.invocationInfo = invocationInfo;
	}

	/**
	 * ʼ߳
	 * 
	 * @param map
	 */
	public void initInvocationInfo() {
		for (Map.Entry<String, Serializable> entry : this.invocationInfo.entrySet()) {
			try {
				String key = entry.getKey();
				String value = String.valueOf(entry.getValue());
				switch (key) {
				case "_UserCode":
					InvocationInfoProxy.getInstance().setUserCode(value);
					break;
				case "_BizCenterCode":
					InvocationInfoProxy.getInstance().setBizCenterCode(value);
					break;
				case "_BizDateTime":
					InvocationInfoProxy.getInstance().setBizDateTime(Long.parseLong(value));
					break;
				case "_BusiAction":
					InvocationInfoProxy.getInstance().setBusiAction(value);
					break;
				case "_CallId":
					InvocationInfoProxy.getInstance().setCallId(value);
					break;
				case "_DeviceId":
					InvocationInfoProxy.getInstance().setDeviceId(value);
					break;
				case "_GroupId":
					InvocationInfoProxy.getInstance().setGroupId(value);
					break;
				case "_GroupNumber":
					InvocationInfoProxy.getInstance().setGroupNumber(value);
					break;
				case "_HyCode":
					InvocationInfoProxy.getInstance().setHyCode(value);
					break;
				case "_LangCode":
					InvocationInfoProxy.getInstance().setLangCode(value);
					break;
				case "_LogLevel":
					InvocationInfoProxy.getInstance().setLogLevel(value);
					break;
				case "_RunAs":
					InvocationInfoProxy.getInstance().setRunAs(value);
					break;
				case "_Sysid":
					InvocationInfoProxy.getInstance().setSysid(value.getBytes()[0]);
					break;
				case "_TimeZone":
					InvocationInfoProxy.getInstance().setTimeZone(value);
					break;
				case "_UserDataSource":
					InvocationInfoProxy.getInstance().setUserDataSource(value);
					break;
				case "_UserId":
					InvocationInfoProxy.getInstance().setUserId(value);
					break;
				case "_token":
					ISecurityTokenCallback stc = NCLocator.getInstance().lookup(ISecurityTokenCallback.class);
					if (!"default".equals(value)) {
						byte[] tokenBytes = KeyUtil.decodeToken(value);
						stc.restore(tokenBytes);
					}
					break;
				case "_SessionId":
					InvocationInfoProxy.getInstance().setProperty("sessionId", value);
					break;
				default:
					break;
				}
			} catch (Throwable e) {
				Logger.error(e);
			}
		}
	}

	@Override
	public Map<String, Object> call() throws Exception {
		initInvocationInfo(); // ʼ߳
		return this.service.tallyBySingleThread(this.pks, this.date, this.op, this.isList);
	}
}
